@@ -1,14 +1,16 @@
 import json
+import os
 from typing import Counter
 import xlwings as xw
 import shutil
 from tqdm import tqdm
 import datetime
 from interval import Interval
+import pandas as pd
 
 class BzttExcl:  #工单处理
     log_lv = 3
-    
+
     app=""
     wb = {}
     # def __init__(self, system_info, oltInfo, soft_ware_info):
@@ -23,14 +25,19 @@ class BzttExcl:  #工单处理
             "FATAL": 6,
             "OFF": 7,
         }
-        log_level = system_info.get("log_level","INFO")
+        log_level = system_info.get("logLevel","INFO")
+        json_title = system_info.get("title","无标题")
+        json_description = system_info.get("description","无描述")
+
         self.log_lv = log_lv_data[log_level]
         print_data = [
-            {"text": "类构造成功", "color": "33"}
+            {"text": "json信息", "color": "33"},
+            {"text": "json标题：" +  json_title, "color": "33"},
+            {"text": "json描述：" +  json_description, "color": "33"},
+            {"text": "配置文件版本：" + system_info["version"], "color": "33"}
         ]
         self.print_log(print_data, "INFO")
 
-        #self.app=xw.App(visible=True,add_book=False)
     def __del__(self):
 
         print ("")
@@ -57,12 +64,16 @@ class BzttExcl:  #工单处理
             "OFF": 7,
         }
         log_lv = log_lv_data[log_lv_num.upper()]
-       
+
         if log_lv >= self.log_lv:
-            print("\033[1;%sm%s\033[0m" % ("34", log_lv_num.upper()), end=" - ")
+            print(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S") + " " + "\033[1;%sm%s\033[0m" % ("34", "[" + log_lv_num.upper() + "]"))
+
             for info in log_lv_info:
                 info.setdefault("end", "\n")
+
                 print("\033[1;%sm%s\033[0m" % (info["color"], info["text"]), end=info["end"])
+
+
     # excel 判断表格内容（ifs）
     def ifsData(self,jsonData):
         screen = "=IFS("
@@ -216,7 +227,84 @@ class BzttExcl:  #工单处理
         shtSource = self.wb[jsonData["sourceTitle"]].sheets(jsonData["sourceShell"])
 
         shtSource.range(jsonData["sourceRange"]["start"],jsonData["sourceRange"]["end"]).api.RemoveDuplicates(int(jsonData["RemoveNum"]))
+    # 初始化数据 （格式化数据）
+    def InitJson(self,formatJson):
+        formatJsonStr =json.dumps(formatJson,ensure_ascii=False)   #转换字符串
 
+        for i in range(0,len(formatJson["formatVariable"])):
+            formatJsonStr =json.dumps(formatJson,ensure_ascii=False)   #转换字符串
+            fucInfoJson = formatJson["formatVariable"]
+            info = fucInfoJson[i]
+            in_value = info["value"]
+
+            if info["type"] == "input":
+                # in_value = {}
+                title = in_value.get("title","请输入：")
+                inputIntStr = input(title)
+                formatJsonStr = formatJsonStr.replace(info["name"],inputIntStr)
+
+            #字符串转换
+            elif info["type"] == "replace":
+                #print("json:",info["value"])
+                in_type = in_value.get("type","str")
+                if in_type == "str":
+                    valueStr = in_value["data"]
+                elif in_type == "json":
+                    valueStr = json.dumps(info["value"]["data"],ensure_ascii=False) 
+                else:
+                    raise Exception("未知替换类型")
+                
+                formatJsonStr = formatJsonStr.replace( "\"" + info["name"] + "\"",valueStr)
+            
+            # 时间替换
+            elif info["type"] == "time":
+
+                in_value = {
+                    "in_data":in_value["inData"],                   #输入时间
+                    "in_format":in_value.get("inFormat","%Y%m%d"), # 输入时间格式,
+                    "out_format":in_value.get("outFormat","%Y%m%d"), # 输出时间格式,
+                    "change_num":in_value.get("changeNum","%Y%m%d"), # 变化数量,
+                    "change_unit":in_value.get("changeUnit","%Y%m%d"), # 变化单位,
+                }
+                target_time_str = self._change_time(in_value)
+                formatJsonStr = formatJsonStr.replace( "\"" + info["name"] + "\"",target_time_str)
+                
+            elif info["type"] == "date2Rang":
+                # print("date2Rang:",info["value"])
+                #str转time
+                SourceTimeStr = datetime.datetime.strptime(str(info["value"]["dateStr"]),info["value"]["dateFormat"])
+                year = SourceTimeStr.year
+                month = SourceTimeStr.month
+                day = SourceTimeStr.day
+                # print("SourceTimeStr",SourceTimeStr,year,month,day)
+                for RangeFormatInfo in info["value"]["RangeFormat"]:
+                    # print("RangeFormatInfo",RangeFormatInfo)
+                    chengeStr = 0
+                    #输出B1
+                    if RangeFormatInfo["type"] == "day":
+                        chengeStr = int(day)
+                    elif RangeFormatInfo["type"] == "month":
+                        chengeStr = int(month)
+                    elif RangeFormatInfo["type"] == "year":
+                        chengeStr = int(year)
+                    zoom_2_5 = Interval(RangeFormatInfo["Range"][0], RangeFormatInfo["Range"][1])
+                    
+                    #在区间内
+                    if chengeStr in zoom_2_5:
+                        if RangeFormatInfo["direction"] == "column":
+                            #字母转转数字 columnInitial
+                            columnInitial = self.Col2Int(RangeFormatInfo["columnInitial"])
+                            columnTial = columnInitial + chengeStr
+                            columnStr = self.ExcelColumn(columnTial - RangeFormatInfo["Range"][0])
+                            chengeAll = columnStr +  RangeFormatInfo["rowInitial"]
+                            # print("chengeAll",chengeAll)
+                            formatJsonStr = formatJsonStr.replace(info["name"],chengeAll)
+                            break
+            #重置
+            formatJson = json.loads(formatJsonStr)
+        data = json.loads(formatJsonStr)
+        return data
+   
     # 复制数据
     def copyData(self,copyJson):
         print_str_copy = "[%s]%s!%s" % (copyJson["sourceTitle"],copyJson["sourceShell"],copyJson["sourceRange"]["start"] + ":" + copyJson["sourceRange"]["end"])
@@ -257,6 +345,7 @@ class BzttExcl:  #工单处理
         if n: ret += self.ExcelColumn(n)
         ret += num[m]
         return ret
+    
     def Col2Int(self,s:str)->int:
         ret=0
         ret += (ord(s[0])-64)*26**(len(s)-1)
@@ -264,57 +353,6 @@ class BzttExcl:  #工单处理
         if s: ret += self.Col2Int(s)
         return ret
 
-    def advancedFilter(self,advancedFilterJson):#高级筛选
-        print("advancedFilterJson",advancedFilterJson)
-        
-        #print("sourceTitle",advancedFilterJson["sourceTitle"])
-        #print("sourceShell",advancedFilterJson["sourceShell"])
-
-        #print("sourceTitle",advancedFilterJson["targetTitle"])
-        #print("sourceShell",advancedFilterJson["targetShell"])
-
-        #数据
-        shtSource = self.wb[advancedFilterJson["sourceTitle"]].sheets(advancedFilterJson["sourceShell"])
-        shtTarget = self.wb[advancedFilterJson["targetTitle"]].sheets(advancedFilterJson["targetShell"])
-        condition = advancedFilterJson["condition"]
-        iNum = 0
-        #获取row
-        info = shtSource.used_range
-        nrows = info.last_cell.row
-        ncolumns = info.last_cell.column
-		#获取表头
-        range_value_list = [shtSource.range((1,1),(1,ncolumns)).value]
-
-        range_value_list_source = shtSource.range(1,1).expand().value
-        for infoData in tqdm(range_value_list_source): #tqdm
-            str_value_row = ""
-            for infoAll in condition:  #符合所有条件  有一个条件不符,则停止
-                for info in infoAll:
-                    if info["type"] == "end":
-                        str_value_row = infoData
-                        break
-                    numId = self.Col2Int(info["Range"]) - 1
-                    select_value_sheet = infoData[numId]
-                    if info["type"] == "disExist":#存在 
-                        if select_value_sheet in info["data"]:
-                            str_value_row = ""
-                            break
-                    elif info["type"] == "exist":#不存在,剔除
-                        
-                        if select_value_sheet not in info["data"]:
-                            break
-                    elif info["type"] == "end":
-                        str_value_row = infoData
-                #数据赋值
-                if str_value_row == "":
-                    continue
-                else:
-                    range_value_list.append(str_value_row)
-                    break
-
-        shtTarget.range(advancedFilterJson["targetRange"]).value = range_value_list
-        self.wb[advancedFilterJson["targetTitle"]].save()
-
     # 格式化数据（初始）
     def formatJson(self,formatJson):
         print_data = [
@@ -339,7 +377,8 @@ class BzttExcl:  #工单处理
             self.print_log(print_data, "DEBUG")
 
             if info["type"] == "input":
-                inputIntStr = input(info["title"])
+                titleTmp = info.get("title","请输入：")
+                inputIntStr = input(titleTmp)
                 #inputIntStr = "20211102"
                 formatJsonStr = formatJsonStr.replace(info["name"],inputIntStr)
 
@@ -388,81 +427,12 @@ class BzttExcl:  #工单处理
                             formatJsonStr = formatJsonStr.replace(info["name"],chengeAll)
                             break
 
-
-
-
-
-
-
-
-
-
-
-
             #重置
             formatJson = json.loads(formatJsonStr)
-
-
-
         data = json.loads(formatJsonStr)
         return data
-            
-
-
-    def advancedFilter_1(self,advancedFilterJson):
-        print("advancedFilterJson",advancedFilterJson)
-
-
-        #数据
-        shtSource = self.wb[advancedFilterJson["sourceTitle"]].sheets(advancedFilterJson["sourceShell"])
-        shtTarget = self.wb[advancedFilterJson["targetTitle"]].sheets(advancedFilterJson["targetShell"])
-        condition = advancedFilterJson["condition"]
-        iNum = 0
-        #获取row
-        info = shtSource.used_range
-        nrows = info.last_cell.row
-        ncolumns = info.last_cell.column
-		#获取表头
-        range_value_list = [shtSource.range((1,1),(1,ncolumns)).value]
-        #获取currn
-        for i in tqdm(range(2,nrows + 1)):
-            #print("数据",i,sht.range("A" + str(i)).value)
-            #判断数据
-            #str_sheet1 = "A"+str(i)+":"+"E"+str(i)
-            str_value_row = ""
-            for infoAll in condition:  #符合所有条件  有一个条件不符,则停止
-                #print("condition:",condition)
-                #print("info:",info)
-                for info in infoAll:
-                    if info["type"] == "disExist":#存在 
-                        #print("执行",iNum)
-                        select_sheet_value = info["Range"]+str(i)
-                        select_value_sheet = shtSource.range(select_sheet_value).value
-                        if select_value_sheet in info["data"]:
-                            str_value_row = ""
-                            break
-                            
-                    elif info["type"] == "exist":#不存在,剔除
-                        select_sheet_value = info["Range"]+str(i)
-                        select_value_sheet = shtSource.range(select_sheet_value).value
-                        if select_value_sheet not in info["data"]:
-                            break
-                    elif info["type"] == "end":
-                        str_value_row = shtSource.range((int(i),1),(int(i),ncolumns)).value
-                    if str_value_row != "":
-                        break     
-                #数据赋值
-                if str_value_row == "":
-                    continue
-                else:
-                    range_value_list.append(str_value_row)
-                    break
-
-
-        #print("data:")
-
-        shtTarget.range(advancedFilterJson["targetRange"]).value = range_value_list
-
+    
+    # 打开文件
     def openfile(self,iniFile):        
         print_data = [
             {"text": "正在复制文件，请稍后...", "color": "33"}
@@ -480,6 +450,7 @@ class BzttExcl:  #工单处理
             self.print_log(print_data, "INFO")
 
             shutil.copyfile(fileInfo["sourcePath"] + fileInfo["name"] + "." + fileInfo["suffix"], fileInfo["targetPath"] + fileInfo["name"] + "_" + fileInfo["title"] + "." + fileInfo["suffix"])
+
         print_data = [
             {"text": "文件复制完毕", "color": "33"}
         ]
@@ -489,9 +460,15 @@ class BzttExcl:  #工单处理
                 {"text": "正在打开文件,请稍后...", "color": "33"},
         ]
         self.print_log(print_data, "INFO")
+        # 格式化app
+        appConfig = iniFile.get("appConfig",{})
+        visibleFlag = appConfig.get("visible",True)
+        add_book = appConfig.get("addBook",False)
+        display_alerts = appConfig.get("displayAlerts",False)
 
-        self.app = xw.App(visible=True,add_book=False)
-        self.app.display_alerts=False
+        self.app = xw.App(visible=visibleFlag,add_book=add_book)
+
+        self.app.display_alerts=display_alerts
         
         #打开文件
         for fileInfo in iniFile["info"]:
@@ -504,12 +481,196 @@ class BzttExcl:  #工单处理
 
             #self.wb[fileInfo["title"]] =  xw.Book(fileInfo["path"] + fileInfo["name"] + "_" + fileInfo["title"] + "." + fileInfo["suffix"]) #创建一个新的Excel文件
             self.wb[fileInfo["title"]] =  self.app.books.open(fileInfo["targetPath"] + fileInfo["name"] + "_" + fileInfo["title"] + "." + fileInfo["suffix"]) #创建一个新的Excel文件
+        
+        
         print_data = [
             {"text": "文件打开完毕", "color": "33"}
         ]
         self.print_log(print_data, "INFO")
         #校验 文件
-        
+ 
+    # 格式化筛选数据
+    def _handle_list_filters(self, args_filters):
+        """
+        处理list接口传入的过滤条件
+        :param args_filters: 传入过滤条件
+        :return: 转换后的sqlalchemy过滤条件
+        """
+        query_str = ""
+        if args_filters:
+            for item in args_filters:
+                query_str = query_str + " and "
+                item_type = item.get("type","str")
+                value = ""
+                # 判断数据类型
+                if item_type == "time": # 转为时间格式
+                    # 2019-06-1
+                    for info in item["value"]:
+                        in_value = {
+                            "in_data":info["data"],
+                            "in_format":info["dataFormat"],
+                            "out_format":"%Y-%m-%d",
+                            "change_num":info["change_num"],
+                            "change_unit":info["change_unit"],
+                        }
+                        value_tmp = self._change_time(in_value)
+                        value = value  +item["range_name"] + info["type"] + "'" + value_tmp + "' and "
+                    value = value[:-4]
+                    pass
+                elif item_type == "str":
+                    value = str(item["value"])
+                    pass
+                elif item_type == "int":
+                    value = int(item["value"])
+                # 生成数据
+                if item["condition"] == '=':
+                    query_str = query_str + item["range_name"] + " == " + value 
+                elif item_type == 'time':
+                    query_str = query_str + value 
+                
+
+        query_str = query_str[4:]
+        return query_str
+    
+    def _getFilterData(self,dfData,inData):
+        for info in inData:
+            range_name = info["range_name"]
+            info_value = info["value"]
+            if info["type"] == "array":
+                if info["condition"] == "in":
+                    dfData = dfData[dfData[info["range_name"]].isin(info_value)]
+                elif info["condition"] == "not in":
+                    dfData = dfData[~dfData[info["range_name"]].isin(info_value)]
+                else:
+                    raise Exception("筛选条件错误")
+            elif info["type"] == "time":
+
+                startTime = self._change_time(info_value[0])
+                endTime = self._change_time(info_value[1])
+                dfData = dfData[(dfData[range_name] >= startTime) & (dfData[range_name] <= endTime)]
+            elif info["type"] == "regex": #正则筛选   
+                if info["condition"] == "in": # 包含
+                    Bool = dfData[range_name].str.contains(info_value)
+                    dfData = dfData[Bool]
+                elif info["condition"] == "not in":# 删除
+                    Bool = dfData[range_name].str.contains(info_value)
+                    dfData = dfData[~Bool]
+                else:
+                    raise Exception("筛选条件错误")
+            else:
+                raise Exception("未知类型。")
+
+        return dfData
+
+        pass
+
+    # 高级筛选  
+    def AdvancedFilter(self,inData):
+        # 写入筛选条件
+        sourceRange = inData["sourceRange"]
+        args_filters = inData["query"]
+        countRange = inData["countRange"] # 统计数量参数
+        sourceSht = self.wb[sourceRange["sourceTitle"]].sheets(sourceRange["sourceShell"])
+        # 读取数据 
+        print_data = [
+            {"text": "正在读取数据，请稍后..." , "color": "33"}
+        ]
+        self.print_log(print_data, "INFO")
+        sourceData=sourceSht.range('A:BW').options(pd.DataFrame,header=1,index=False,expand='table').value
+        print_data = [
+            {"text": "读取成功，正在处理数据..." , "color": "33"}
+        ]
+        self.print_log(print_data, "INFO")
+        # 生成筛选字符串
+        df_data_all=pd.DataFrame([])
+        query_str_all = ""
+        # 循环生成数据
+        for info in args_filters:
+            df_data_tmp = self._getFilterData(sourceData,info)
+            df_data_all=pd.concat([df_data_all,df_data_tmp])
+        # 删除重复项
+        df_data_all = df_data_all.drop_duplicates(keep="first")
+        print_data = [
+            {"text": "处理成功，筛选出 " + str(len(df_data_all)) + " 条数据" , "color": "33"},
+            {"text": "正在记录明细... ", "color": "33"}
+        ]
+        self.print_log(print_data, "INFO")
+        # 循环写入明细数据
+        for details_range_info in inData["detailsRange"]:
+            df_data_all_columns = details_range_info.get("columns",df_data_all.columns)
+            self.wb[details_range_info["targetTitle"]].sheets(details_range_info["targetShell"]). \
+                range(details_range_info["targetRange"]["startLetter"] + str(details_range_info["targetRange"]["startNum"])) \
+                    .value=df_data_all[df_data_all_columns].set_index(df_data_all_columns[0])
+
+        print_data = [
+            {"text": "明细记录成功 " , "color": "33"},
+            {"text": "正在计算每种数量... ", "color": "33"}
+        ]
+        self.print_log(print_data, "INFO")
+
+        # 循环计算数量
+        for countRange_info in countRange:  
+            # 分别计算区县数量
+            countyQuery = countRange_info["countyQuery"]  # 区县筛选条件
+            countyArrayTmp = []
+            show_info = []
+            for countyQuery_info in countyQuery: # 分别计算区县数量
+                numData = str(len(self._getFilterData(df_data_all,countyQuery_info["value"])))
+                countyArrayTmp = countyArrayTmp + [numData]
+                show_one = {countyQuery_info.get("title","无"):numData}
+                show_info  = show_info +[show_one]
+                pass
+            # 写入内容 
+            print_data = [
+                {"text": "统计数量结果 : " + str(show_info), "color": "33"},
+            ]
+            self.print_log(print_data, "INFO")
+
+            directionTmp = countRange_info.get("direction","column")
+            if directionTmp =="row":
+                flagTmp = False
+            elif directionTmp =="column":
+                flagTmp = True
+
+            self.wb[countRange_info["targetTitle"]].sheets(countRange_info["targetShell"]). \
+                range(countRange_info["targetRange"]["startLetter"] + str(countRange_info["targetRange"]["startNum"])). \
+                        options(transpose=flagTmp).value=countyArrayTmp
+
+    # 更改时间
+    def _change_time(self,in_value):
+        startTimeStr = str(in_value["in_data"]) # 输入时间
+        in_format = in_value.get("in_format","%Y%m%d") # 输入时间格式
+        out_format = in_value.get("out_format","%Y%m%d") # 输出时间格式
+        change_num = int(in_value.get("change_num",0)) # 变化数量
+        change_unit = in_value.get("change_unit","day") # 变化单位
+
+        if change_unit == "day":
+            SourceTimeStr = datetime.datetime.strptime(startTimeStr,in_format)  + datetime.timedelta(days = change_num)
+        else:
+            raise Exception("未知变化类型")
+        startTimeStr = SourceTimeStr.strftime( out_format )   #'>%Y-%m-%d %H:%M:%S'
+        return startTimeStr
+
+    # 整体执行操作
+    def performOperate(self,operate_json):
+        for info in operate_json:
+            if info["type"] == "advancedFilter": # 高级筛选
+                print_data = [
+                    {"text": "执行高级筛选：" + info.get("title","无描述"), "color": "33"}
+                ]
+                self.print_log(print_data, "INFO")
+                self.AdvancedFilter(info["value"])
+            if info["type"] == "screening":
+                self.countyCountyNum(info)
+            if info["type"] == "copy":
+                self.copyData(info)
+            if info["type"] == "ifs":
+                self.ifsData(info)
+            if info["type"] == "addShell":
+                self.addShell(info)
+            if info["type"] == "RemoveDuplicates":
+                self.RemoveDuplicates(info)
+        pass
 
    
 
